#include "lmbf.h"
// cumbia
#include <cumbiapool.h>
#include <cuserviceprovider.h>
#include <cumacros.h>
#include <quapps.h>
#include <quplotcurve.h>
// cumbia

#include <qwt_plot_item.h>
#include "ui_lmbf.h"
#include <QDebug>

LmbfGui::LmbfGui(CumbiaPool *cumbia_pool, QWidget *parent) :
    QWidget(parent) {
// cumbia
    CuModuleLoader mloader(cumbia_pool, &m_ctrl_factory_pool, &m_log_impl);
    cu_pool = cumbia_pool;
    ui = new Ui::LmbfGui;
    ui->setupUi(this, cu_pool, m_ctrl_factory_pool);


    // plot options
    ui->tPlotLightMarkerADC->setOptions(CuData("period", 500));
    ui->tPlotLightMarkerFFt->setOptions(CuData("period", 500));
    ui->tPlotLightMarkerADC->setSource("$3/I_ADC_MMS_DELTA");
    ui->tPlotLightMarkerFFt->setSource("lmbf/buf/0/FFTADC0Avg");

    fftScale = new fftScaleDraw();
    ui->tPlotLightMarkerFFt->setAxisScaleDraw(QwtPlot::xBottom, fftScale);
    ui->tPlotLightMarkerFFt->axisScaleEngine(QwtPlot::xBottom)->setBase(2);

    num_turns=1024; //fake value for starting

    normalpalette = ui->tLabel_7->palette();
    alarmpalette  = ui->tLabel_7->palette();
    normalpalette.setColor(QPalette::Background, QColor(Qt::green));
    alarmpalette.setColor(QPalette::Background, QColor(Qt::red));

    turns_reader = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
    turns_reader->attach(this, SLOT(update_num_turns(double)));
    turns_reader->setSource("lmbf/buf/0/numturns");

    sevr_reader = new QuWatcher(this, cumbia_pool, m_ctrl_factory_pool);
    sevr_reader->attach(this, SLOT(update_sevr(QString)));
    sevr_reader->setSource("$3/IQ_STA_SEVR");
    
    QwtText au=QwtText("Amplitude [a.u]");
    au.setFont(QFont("Helvetica",9));

    ui->tPlotLightMarkerFFt->setAxisTitle(QwtPlot::yLeft, au);
    ui->tPlotLightMarkerADC->setAxisTitle(QwtPlot::yLeft, au);
    ui->tPlotLightMarkerModesUp->setAxisTitle(QwtPlot::yLeft, au);
    ui->tPlotLightMarkerModesDown->setAxisTitle(QwtPlot::yLeft, au);
    
    QwtText instab=QwtText("Bunch instability");
    instab.setFont(QFont("Helvetica",10));
    ui->tPlotLightMarkerADC->setTitle(instab);
    
    QwtText spectr=QwtText("Average bunch spectrum");
    spectr.setFont(QFont("Helvetica",10));
    ui->tPlotLightMarkerFFt->setTitle(spectr);
    
    QwtText lmodes=QwtText("Longitudinal modes");
    lmodes.setFont(QFont("Helvetica",10));
    ui->tPlotLightMarkerModesUp->setTitle(lmodes);
    
    QwtText modelegend=QwtText("mode number");
    modelegend.setFont(QFont("Helvetica",9));
    ui->tPlotLightMarkerModesDown->setAxisTitle(QwtPlot::xBottom, modelegend);
    
    QwtText bnum=QwtText("bunch number");
    bnum.setFont(QFont("Helvetica",9));
    ui->tPlotLightMarkerADC->setAxisTitle(QwtPlot::xBottom, bnum);
    
    QwtText fractune=QwtText("fractional tune");
    fractune.setFont(QFont("Helvetica",9));
    ui->tPlotLightMarkerFFt->setAxisTitle(QwtPlot::xBottom, fractune);

    ui->tPlotLightMarkerFFt->setAxisLabelRotation(QwtPlot::xBottom, 30);
    ui->tPlotLightMarkerFFt->setAxisLabelAlignment(QwtPlot::xBottom, Qt::AlignRight);

    connect(ui->tPlotLightMarkerADC, SIGNAL(curveAdded(QuPlotCurve *)), this, SLOT(setFillArea(QuPlotCurve *)));
    connect(ui->tPlotLightMarkerModesUp, SIGNAL(curveAdded(QuPlotCurve *)), this, SLOT(setCurveSticksStyle(QuPlotCurve *)));
    connect(ui->tPlotLightMarkerModesDown, SIGNAL(curveAdded(QuPlotCurve *)), this, SLOT(setCurveSticksStyle(QuPlotCurve *)));

    // scale change
    foreach(QPushButton *pb, findChildren<QPushButton *>(QRegularExpression("pbscale.*")))
        connect(pb, SIGNAL(clicked()),this , SLOT(scale_change()));
} 

LmbfGui::~LmbfGui()
{
    delete turns_reader;
    delete ui;
}

void LmbfGui::update_num_turns(double nturns)
{
    qDebug() << "update_num_turns " << nturns ;
    fftScale->set_num_turns(nturns);
    if (nturns > 0 && nturns != num_turns){
        num_turns = nturns;
        int bound= static_cast<int>(num_turns/20);
        ui->tPlotLightMarkerFFt->setXUpperBound( bound );
        ui->tPlotLightMarkerFFt->setAxisScale(QwtPlot::xBottom, 0.0, bound, 0);
        ui->tPlotLightMarkerFFt->axisScaleEngine(QwtPlot::xBottom)->setBase(2);
    }
}


fftScaleDraw::fftScaleDraw()
{
    num_turns=1; //trick to reuse the slot during initalization
    double initial=1024.0; //guess value
    set_num_turns(initial);
}

void fftScaleDraw::set_num_turns(double n)
{
    qDebug() << "set_num_turns " << n ;
    if (n> 0 && n!=num_turns){
        num_turns = n;
    }
}

fftScaleDraw::~fftScaleDraw()
{
    fftLabels.clear();
}

QwtText fftScaleDraw::label(double v) const{
    qDebug() << "label "<<num_turns <<" "<< v << " " <<  (v+1)/(num_turns);
    return QString::number((v+1)/(num_turns) ,'e', 3);
}

void LmbfGui::update_sevr(QString sevr)
{
    if (sevr == "NO_ALARM"){
        ui->tLabel_7->setPalette(normalpalette);
        ui->tLabel_7->setText("OK");
    }
    else{
        ui->tLabel_7->setPalette(alarmpalette);
        ui->tLabel_7->setText("NOT OK");
    }
}

void LmbfGui::setCurveSticksStyle(QuPlotCurve *c) {
    c->setStyle(QwtPlotCurve::Sticks);
}

void LmbfGui::setFillArea(QuPlotCurve *c) {
    c->setBrush(QBrush(c->pen().color().lighter()));
    c->setBaseline(0.0);
}

void LmbfGui::scale_change() {
    QuPlotBase *p;
    const QString& s = sender()->objectName();
    bool up = s.contains("Up", Qt::CaseInsensitive);
    if(s.contains("ADC")) p = ui->tPlotLightMarkerADC;
    else p = ui->tPlotLightMarkerFFt;
    double ub = p->yUpperBound();
    // assume lower bound zero
    up ? ub = 2 * ub : ub = ub / 2.0;
    p->setYUpperBound(ub);
}


